@@ -1,6 +1,6 @@
 """Functions that use pandas to read transcript information."""
 
-import warnings
+import logging
 
 from Bio import SeqIO
 from Bio.Seq import Seq
@@ -22,8 +22,8 @@ def _check_column_presence(data_frame, column, message=""):
 def _check_column_absence(data_frame, column, message=""):
     """Give a warning if the column is already present in the data_frame."""
     if column in data_frame.columns:
-        warnings.warn("Input DataFrame already has column: %s. %s" %
-                      (column, message))
+        logging.warning("Input DataFrame already has column: %s. %s", column,
+                        message)
 
 
 def _get_flag(flag):
@@ -308,6 +308,9 @@ def _manage_start_phase_negative_strand(cds_seq, start_phase):
     >>> _manage_start_phase_negative_strand('XYYY', 2)
     'YYY'
     """
+    if cds_seq is None:
+        return None
+
     if start_phase == 1:
         return cds_seq[2:]
     if start_phase == 2:
@@ -332,17 +335,59 @@ def _manage_end_phase_negative_strand(row_list, row_index, cds_seq, end_exon):
     to ensure that there are not errors in the input data.
     """
     if cds_seq is None:
-        return None
+        return None, False
     end_phase = row_list[row_index]['EndPhase']
+    skip = 0
     if end_phase in {1, 2}:
         if end_exon:
             cds_seq = cds_seq[:-end_phase]  # delete incomplete codon
         else:
-            next_exon_sequence = row_list[row_index + 1]['ExonSequence']
             n_bases = phases.bases_to_complete_previous_codon(end_phase)
-            cds_seq = cds_seq + next_exon_sequence[0:n_bases]
+            cds_seq, skip = _take_n_bases(cds_seq, n_bases, 'next', row_list,
+                                          row_index)
 
-    return cds_seq
+    return cds_seq, skip
+
+
+def _take_n_bases(cds_seq, nbases, direction, row_list, row_index):
+    """
+    Helper function to deal with shared codons in (sub-)exon boundaries.
+    """
+    rows = []
+    while 0 <= row_index < len(row_list):
+        rows.append(row_list[row_index])
+        if direction == "next":
+            row_index += 1
+        else:
+            row_index -= 1
+    seqs = [row['ExonSequence'] for row in rows]
+    lengths = [len(seq) for seq in seqs]
+    trxs = [row['TranscriptID'] for row in rows]
+    cds = [cds_seq]
+    idx = 1
+    skip = 0
+    while nbases and idx < len(rows):
+
+        if trxs[idx] != trxs[idx - 1]:
+            return None, 0
+
+        if lengths[idx] > nbases:
+            if direction == "next":
+                cds.append(seqs[idx][0:nbases])
+            else:
+                seq = seqs[idx]
+                cds.insert(0, seq[len(seq) - nbases:])
+            nbases = 0
+        else:
+            nbases -= lengths[idx]
+            if direction == "next":
+                cds.append(seqs[idx])
+            else:
+                cds.insert(0, seqs[idx])
+            idx += 1
+            skip += 1
+
+    return ''.join(str(seq) for seq in cds), skip
 
 
 def _manage_start_phase_positive_strand(row_list, row_index, cds_seq,
@@ -362,15 +407,18 @@ def _manage_start_phase_positive_strand(row_list, row_index, cds_seq,
     >>> seq_one = 'AATTGATATATAAGGAAGTTATGGACTTGGAAGAGAGAACCAAGAATGGAGTCATACGG'
     >>> seq_one = seq_one + 'GGGCAGCCTTCTCCTTTAGGTTG'
     >>> seq_two = 'TCAAAAGACACATTCTGTAGTGTTGTAA'
-    >>> row_one = {'StartPhase': 1, 'EndPhase': 2, 'ExonSequence': seq_one}
-    >>> row_two = {'StartPhase': 2, 'EndPhase': 0, 'ExonSequence': seq_two}
+    >>> row_one = {'StartPhase': 1, 'EndPhase': 2, 'ExonSequence': seq_one, \
+            'TranscriptID': ''}
+    >>> row_two = {'StartPhase': 2, 'EndPhase': 0, 'ExonSequence': seq_two, \
+            'TranscriptID': ''}
     >>> row_list = [row_one, row_two]
     >>> _manage_start_phase_positive_strand(row_list, 1, seq_two, False)
-    'TGTCAAAAGACACATTCTGTAGTGTTGTAA'
+    ('TGTCAAAAGACACATTCTGTAGTGTTGTAA', 0)
     """
     if cds_seq is None:
-        return None
+        return None, False
     start_phase = row_list[row_index]['StartPhase']
+    skip = 0
     if start_phase in {1, 2}:
         if start_exon:
             n_bases = phases.bases_to_complete_previous_codon(start_phase)
@@ -379,11 +427,10 @@ def _manage_start_phase_positive_strand(row_list, row_index, cds_seq,
             n_bases = phases.bases_to_complete_next_codon(start_phase)
             if n_bases is None:
                 return None
-            seq = row_list[row_index - 1]['ExonSequence']
-            seq = seq[-n_bases:]  # pylint: disable=invalid-unary-operand-type
-            cds_seq = seq + cds_seq
+            cds_seq, skip = _take_n_bases(cds_seq, n_bases, 'previous',
+                                          row_list, row_index)
 
-    return cds_seq
+    return cds_seq, skip
 
 
 def _manage_end_phase_positive_strand(cds_seq, end_phase):
@@ -425,17 +472,20 @@ def _check_phases_by_position(row, end_exon, allow_incomplete_cds):
     The pandas' DataFrame row is printed in the error message.
     """
     end_phase = row['EndPhase']
-    if end_exon:
-        if not allow_incomplete_cds and end_phase in {1, 2}:
-            raise ValueError('Exon end phase is 1 or 2 in last exon, '
-                             'row:  %s' % row)
-    else:
-        if end_phase == -1:
-            if row['StartPhase'] != -1:
-                raise ValueError('Exon end phase -1 is not in the last exon, '
-                                 'row: {}'.format(row))
-            warnings.warn('Exon start and end phases are -1, possible '
-                          'unique exon in row: {}'.format(row))
+    transcript_id = row['TranscriptID']
+    exon_id = row['ExonID']
+
+    if end_exon and (not allow_incomplete_cds) and end_phase in {1, 2}:
+        logging.warning('Exon end phase is 1 or 2 in last exon (%s, %s).',
+                        transcript_id, exon_id)
+        return False
+
+    if (not end_exon) and end_phase == -1:
+        logging.warning('Exon end phase -1 is not in the last exon (%s, %s).',
+                        transcript_id, exon_id)
+        return False
+
+    return True
 
 
 def _is_incomplete_cds(row, start_exon, end_exon):
@@ -451,9 +501,10 @@ def _is_incomplete_cds(row, start_exon, end_exon):
     return False
 
 
-def add_protein_seq(data_frame,
-                    allow_incomplete_cds=True,
-                    seq_column="ExonProteinSequence"):
+def add_protein_seq(  # pylint: disable=too-many-branches,too-many-locals,too-many-statements
+        data_frame,
+        allow_incomplete_cds=True,
+        seq_column="ExonProteinSequence"):
     """
     Add a column with the protein sequence of the exon.
 
@@ -468,13 +519,18 @@ def add_protein_seq(data_frame,
     _check_column_absence(data_frame, 'IncompleteCDS',
                           'Values are going to change.')
 
-    sequences = []
-    incomplete_cds = []
-
     row_list = data_frame.to_dict('records')
     n_rows = len(row_list)
+    sequences = ['' for _ in range(0, n_rows)]
+    incomplete_cds = [False for _ in range(0, n_rows)]
     row_index = 0
+    skip_prev = 0
+    skip_next = 0
     while row_index < n_rows:
+        if skip_next:
+            skip_next -= 1
+            row_index += 1
+            continue
         start_exon, end_exon = _is_first_or_last_exon(row_list, row_index)
 
         # Copy of the actual row to have easier access :
@@ -505,38 +561,51 @@ def add_protein_seq(data_frame,
             # the exon at the right for the positive strand (i.e. to the
             # exon that changes because of the phase change):
 
-            _check_phases_by_position(row, end_exon, allow_incomplete_cds)
+            if not _check_phases_by_position(row, end_exon,
+                                             allow_incomplete_cds):
+                cds_seq = None
 
             if row['Strand'] == -1:
                 cds_seq = _manage_start_phase_negative_strand(
                     cds_seq, row['StartPhase'])
-                cds_seq = _manage_end_phase_negative_strand(
+                cds_seq, skip_next = _manage_end_phase_negative_strand(
                     row_list, row_index, cds_seq, end_exon)
             else:
                 cds_seq = _manage_end_phase_positive_strand(
                     cds_seq, row['EndPhase'])
-                cds_seq = _manage_start_phase_positive_strand(
+                cds_seq, skip_prev = _manage_start_phase_positive_strand(
                     row_list, row_index, cds_seq, start_exon)
 
+        if skip_prev:
+            for skip in range(1, skip_prev + 1):
+                sequences[row_index - skip] = ""
+            skip_prev = 0
+
         # Look for signals of incomplete CDS :
-        incomplete = _is_incomplete_cds(row, start_exon, end_exon)
-        if (cds_seq is not None) and ((len(cds_seq) % 3) == 0):
-            # Add the translated CDS :
-            if isinstance(cds_seq, str):
-                sequences.append(Seq(cds_seq, IUPAC.extended_dna).translate())
+        incomplete = True if cds_seq is None else _is_incomplete_cds(
+            row, start_exon, end_exon)
+
+        if not incomplete:
+            if (len(cds_seq) % 3) == 0:
+                # Add the translated CDS :
+                if isinstance(cds_seq, str):
+                    sequences[row_index] = Seq(cds_seq,
+                                               IUPAC.extended_dna).translate()
+                else:
+                    sequences[row_index] = cds_seq.translate()
             else:
-                sequences.append(cds_seq.translate())
-        else:
-            sequences.append("")
-            incomplete = True
-        incomplete_cds.append(incomplete)
+                incomplete = True
+
+        if seq_column != "SubexonProteinSequence":
+            incomplete_cds[row_index] = incomplete
 
         row_index += 1
 
     # Add new column to store the sequences :
     data_frame[seq_column] = sequences
     # New column to store signals of incomplete CDS :
-    data_frame['IncompleteCDS'] = incomplete_cds
+    if seq_column != "SubexonProteinSequence":  # do not add it for sub-exons
+        data_frame['IncompleteCDS'] = incomplete_cds
 
 
 def _different_phases(row_i, row_j):
@@ -631,7 +700,19 @@ def find_identical_exons(data_frame,
     return exon_clusters
 
 
-def delete_incomplete_sequences(data_frame):
+def _without_stopcodon(sequences):
+    """
+    Return True for the sequences that doesn't end with '*'.
+    """
+    sequence = "".join([str(seq) for seq in sequences])
+    if sequence:
+        return sequence[-1] != "*"
+    return True
+
+
+def delete_incomplete_sequences(data_frame,
+                                seq_column="ExonProteinSequence",
+                                trx_column="TranscriptID"):
     """
     Delete incomplete sequences in place.
 
@@ -640,29 +721,27 @@ def delete_incomplete_sequences(data_frame):
     It also deletes sequences that probably has an incomplete CDS because
     starts or ends with phases differente fron 0 or -1.
     """
-    _check_column_presence(data_frame, 'ExonProteinSequence',
+    _check_column_presence(data_frame, seq_column,
                            'You need to run add_protein_seq first.')
 
-    incomplete_cdss = data_frame.loc[:, ['TranscriptID', 'IncompleteCDS'
-                                         ]].groupby('TranscriptID').agg(
-                                             lambda df: sum(df) > 0)
+    incomplete_cdss = data_frame.loc[:, [trx_column, 'IncompleteCDS']].groupby(
+        trx_column).agg(lambda df: sum(df) > 0)
 
-    incomplete_seqs = data_frame.loc[:, ['TranscriptID',
-                                         'ExonProteinSequence']]. \
-        groupby('TranscriptID'). \
-        agg(lambda df: "".join([str(s) for s in df])[-1] != "*")
-    # Incomplete sequences do not end with '*'
+    incomplete_seqs = data_frame.loc[:, [trx_column,
+                                         seq_column]]. \
+        groupby(trx_column). \
+        agg(_without_stopcodon)  # Incomplete sequences do not end with '*'
 
     if not incomplete_seqs.empty:
-        incomplete_transcripts = set(incomplete_seqs.index[
-            incomplete_seqs['ExonProteinSequence']]).union(
+        incomplete_transcripts = set(
+            incomplete_seqs.index[incomplete_seqs[seq_column]]).union(
                 incomplete_cdss.index[incomplete_cdss['IncompleteCDS']])
         if incomplete_transcripts:
             clusters.inform_about_deletions(
                 incomplete_transcripts, "Incomplete transcripts were found:")
             data_frame.drop([
                 i for i in data_frame.index
-                if data_frame.loc[i, 'TranscriptID'] in incomplete_transcripts
+                if data_frame.loc[i, trx_column] in incomplete_transcripts
             ],
                             inplace=True)
 
